/*
 * An application to echo what is sent to it.  
 *
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>

#include <netdb.h>
#include <signal.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "buffer.h"
#include "iounix.h"

void f_die(char *);
void sigchld_handler(int);

#define BACKLOG 10
#define PORT "8007"
#define BUFSIZE 4096 /* Size to use for buffer. */

void f_die(char *p_text)
{
  fprintf(stderr, "%s", p_text);
  exit(1);
}

void sigchld_handler(int s)
{
  while (waitpid(-1, NULL, WNOHANG) > 0)
    ;
}

int main(void)
{
  struct addrinfo hints, *res;
  struct sockaddr_storage addr;
  struct sigaction sa;
  socklen_t addrlen = sizeof(addr);
  pid_t pid_ret;
  int lfd, afd;
  int int_ret;
  
  /* Grab any address in a AF-independent way. Also gets port. */
  memset(&hints, 0, sizeof(hints));
  hints.ai_family = PF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;
  hints.ai_flags = AI_PASSIVE;
  int_ret = getaddrinfo(NULL, PORT, &hints, &res);
  if (int_ret != 0)
    f_die("main: getaddrinfo: failed.");
  
  /* Now bind and listen on the address and port in res. */
  lfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
  if (lfd < 0)
    f_die("main: socket: failed.");
  if (bind(lfd, res->ai_addr, res->ai_addrlen) < 0) {
    close(lfd);
    f_die("main: bind: failed.");
  }
  if (listen(lfd, BACKLOG) < 0) {
    close(lfd);
    f_die("main: listen: failed.");
  }

  freeaddrinfo(res); /* Must free memory */

  /* Set up signal handler to reap dead processes. */
  sa.sa_handler = sigchld_handler;
  sigemptyset(&sa.sa_mask);
  sa.sa_flags = SA_RESTART;
  if (sigaction(SIGCHLD, &sa, NULL) < 0)
    f_die("sigaction: failed.");

  while (1) { /* TAG: accept loop */
    if ((afd = accept(lfd, (struct sockaddr *)&addr, &addrlen)) < 0) {
      perror("accept");
      continue;
    }

    pid_ret = fork();
    
    if (pid_ret == 0) { /* TAG: child process */ 
      p_arr buf;
      size_t size;
      ssize_t ssize_ret;
      
      close(lfd); /* We don't need it */

      p_buf_init_static(&buf, BUFSIZE);
      
      while (1) {
        if ((ssize_ret = p_io_read(&buf, afd, BUFSIZE)) < 0) /* Error */
          f_die("main: p_io_read: failed.");
        else if (ssize_ret == 0) /* Finished reading (connection terminated) */
          break;
        size = (size_t) ssize_ret;
        if (p_io_write_all(afd, &buf, &size) != 1) /* Not wrote all. */
          f_die("main: p_io_write_all: failed.");
        p_buf_empty(&buf);
      }
      close(afd);
      _exit(0);
    } /* END: TAG: child process */ else if (pid_ret < 0) { /* Error */
      perror("fork");
      continue;
    }
  } /* END: TAG: accept loop */

  /* NOTREACHED */
  f_die("main: should never reach here.");
}
