/*
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef P_NET_H
#define P_NET_H

#include "arr.h"
#include "returns.h"

#define P_NET_RET_INTFAIL -2
#define P_NET_RET_EXTFAIL -1
#define P_NET_RET_PARTSENT 0
#define P_NET_RET_SENT 1

/* PURPOSE
 * To send to p_sock p_size of p_buf.
 * PARAMATERS
 * p_sock       - Socket file descriptor.
 * p_buf        - Pointer to p_arr structure.
 * p_flags      - Flags to pass on to send().
 * p_size       - Number of bytes to send.
 * RETURN VALUES
 * -2 (P_NET_RET_INTFAIL) on internal failure, -1 (P_NET_RET_EXTFAIL) on
 * external failure and number of bytes sent on success. */

ssize_t p_net_send(int p_sock, p_arr *p_buf, int p_flags, size_t p_size);

/* PURPOSE
 * To send to p_sock p_size of p_buf doing it's best to send all of p_size.
 * PARAMATERS
 * p_sock       - Socket file descriptor.
 * p_buf        - Pointer to p_arr structure.
 * p_flags      - Flags to pass on to send().
 * p_size       - Pointer to number of bytes to send.
 * RETURN VALUES
 * -2 (P_NET_RET_INTFAIL) on internal failure, -1 (P_NET_RET_EXTFAIL) on
 * external failure, 0 (P_NET_RET_PARTSENT) on partial send and 
 * 1 (P_NET_RET_SENT) on complete send. What p_size points to is updated
 * to be the number of bytes sent. */

int p_net_send_all(int p_sock, p_arr *p_buf, int p_flags, size_t *p_size);

/* PURPOSE
 * To send to p_sock p_size of p_buf with address of p_to.
 * PARAMATERS
 * p_sock       - Socket file descriptor.
 * p_buf        - Pointer to p_arr structure.
 * p_to         - Pointer to structure sockaddr containing address.
 * p_tolen      - Size of what p_to points to.
 * p_flags      - Flags to pass on to send().
 * p_size       - Number of bytes to send.
 * RETURN VALUES
 * -2 (P_NET_RET_INTFAIL) on internal failure, -1 (P_NET_RET_EXTFAIL) on
 * external failure and number of bytes sent on success. */

int p_net_sendto(int p_sock, p_arr *p_buf, const struct sockaddr *p_to,
                 socklen_t p_tolen, int p_flags, size_t p_size);

/* PURPOSE
 * To send to p_sock p_size of p_buf with address of p_to doing it's best to
 * send all of p_size.
 * PARAMATERS
 * p_sock       - Socket file descriptor.
 * p_buf        - Pointer to p_arr structure.
 * p_to         - Pointer to structure sockaddr containing address.
 * p_tolen      - Size of what p_to points to.
 * p_flags      - Flags to pass on to send().
 * p_size       - Pointer to number of bytes to send.
 * RETURN VALUES
 * -2 (P_NET_RET_INTFAIL) on internal failure, -1 (P_NET_RET_EXTFAIL) on
 * external failure, 0 (P_NET_RET_PARTSENT) on partial send and 
 * 1 (P_NET_RET_SENT) on complete send. What p_size points to is updated
 * to be the number of bytes sent. */

int p_net_sendto_all(int p_sock, p_arr *p_buf, const struct sockaddr *p_to,
                     socklen_t p_tolen, int p_flags, size_t *p_size);

/* PURPOSE
 * To append to p_buf p_size bytes recieved from p_sock.
 * PARAMATERS
 * p_buf        - Pointer to p_arr structure.
 * p_sock       - Socket file descriptor.
 * p_flags      - Flags to pass on to recv().
 * p_size       - Number of bytes to receive.
 * RETURN VALUES
 * Number of bytes received on success, -2 (P_NET_RET_INTFAIL) on internal
 * failure, -1 (P_NET_RET_EXTFAIL) on external failure. */

ssize_t p_net_recv(p_arr *p_buf, int p_sock, int p_flags, size_t p_size);

/* PURPOSE
 * To append to p_buf p_size bytes received from p_sock from address p_from.
 * PARAMATERS
 * p_buf        - Pointer to p_arr structure.
 * p_sock       - Socket file descriptor.
 * p_from       - Pointer to structure sockaddr containing address.
 * p_fromlen    - Size of what p_from points to.
 * p_flags      - Flags to pass on to recvfrom().
 * p_size       - Number of bytes to receieve.
 * RETURN VALUES
 * Number of bytes receieved on success, -2 (P_NET_RET_INTFAIL) on internal
 * failure, -1 (P_NET_RET_EXTFAIL) on external failure. */

ssize_t p_net_recvfrom(p_arr *p_buf, int p_sock, struct sockaddr *p_from,
                       socklen_t *p_fromlen, int p_flags, size_t p_size);

#endif /* P_NET_H */
